package lightcontainer.protocol.rules.reader;

import lightcontainer.domains.client.Context;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.ProtocolReader;
import lightcontainer.protocol.rules.writer.GetFileOkRule;
import lightcontainer.protocol.rules.writer.SaveFileErrorRule;
import lightcontainer.utils.FileReceiver;
import lightcontainer.utils.FileSender;

import java.io.InputStream;

public class RetrieveOkRule extends ProtocolReader {
    // Constants
    private static final String PATTERN = "^RETRIEVE_OK ([A-Za-z0-9.]{50,200}) ([0-9]{1,10}) ([A-Za-z0-9.]{50,200})\r\n$";
    private static final String NAME = "RETRIEVE_OK";
    // -- arguments
    private static final int HASHED_FILE_NAME = 0; // Index hashed filename
    private static final int FILE_SIZE = 1; // Index file size
    private static final int HASHED_FILE_CONTENT = 2; // Index hashed file content

    // Variables
    private ProtocolRepository protocolRep;
    private String storagePath;

    // Variables
    public RetrieveOkRule(ProtocolRepository protocolRep, String storagePath) {
        super(NAME, PATTERN);
        this.protocolRep = protocolRep;
        this.storagePath = storagePath;
    }

    public class Result extends ProtocolResult {
        // Variables
        private String filename;
        private int filesize;
        private String hashedFileContent;

        public Result(Context context, String filename, int filesize, String hashedFileContent) {
            super(context);
            this.filename = filename;
            this.filesize = filesize;
            this.hashedFileContent = hashedFileContent;
        }

        public String getFilename() {
            return filename;
        }

        public int getFilesize() {
            return filesize;
        }

        /**
         * Bitch has bettern than my money
         * @param reader Buffer rempli du fichier
         */
        @Override
        public void read(InputStream reader) {
            super.read(reader);
            System.out.println("Récupération du fichier du SBE");

            FileReceiver fileReceiver = new FileReceiver(storagePath);
            fileReceiver.receiveFile(reader, this.filename, this.filesize);

            // TODO fingerprint <!!!!!!>
        }
    }

    /**
     * Cette méthode est appelée lors de l'exécution de la règle
     *
     * @param context Utilisateur context.
     * @param data    Paramètres pour créer la commande.
     */
    @Override
    protected RetrieveOkRule.Result onExecuted(Context context, String... data) {
        RetrieveOkRule.Result result = new RetrieveOkRule.Result(context, data[HASHED_FILE_NAME], Integer.parseInt(data[FILE_SIZE]), data[HASHED_FILE_CONTENT]);

        // save encrypted file size into bundle
        context.putDataInt("encryptedFileSize", result.getFilesize()); // TODO to long ?!
        context.putDataString("hashedFileName", result.getFilename());

        // Set result command
        result.setResultCommand(protocolRep.executeWriter(context, GetFileOkRule.NAME, context.getDataString("fileName"), String.valueOf(context.getDataInt("fileSize"))), ResultCmdReceiver.CLIENT);
        return result;
    }

    @Override
    protected ProtocolReader.ProtocolResult onError(Context context) {
        ProtocolResult result = new ProtocolResult(context);
        result.setResultCommand(protocolRep.executeWriter(context, SaveFileErrorRule.NAME), ResultCmdReceiver.CLIENT);
        return result;
    }
}
